#
# Cookbook Name:: owncloud
# Recipe:: default
#
# Copyright 2013, Onddo Labs, Sl.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#==============================================================================
# Calculate dependencies for different distros
#==============================================================================

dbtype = node['owncloud']['config']['dbtype']

case node['platform']
when 'debian', 'ubuntu'
  # Sync apt package index
  include_recipe 'apt'

  php_pkgs = %w{ php5-gd php5-intl php5-curl smbclient }
  php_pkgs << 'php5-sqlite' if dbtype == 'sqlite'
  php_pkgs << 'php5-mysql' if dbtype == 'mysql'
  php_pkgs << 'php5-pgsql' if dbtype == 'pgsql'

  ssl_key_dir = '/etc/ssl/private'
  ssl_cert_dir = '/etc/ssl/certs'
when 'redhat', 'centos'
  if node['platform_version'].to_f < 6
    php_pkgs = %w{ php53-gd php53-mbstring php53-xml php53-intl samba-client }
    php_pkgs << 'php53-mysql' if dbtype == 'mysql'
    php_pkgs << 'php53-pgsql' if dbtype == 'pgsql'
    if dbtype == 'sqlite'
      Chef::Application.fatal!("SQLite database type not supported on #{node['platform']} 5.")
    end
  else
    php_pkgs = %w{ php-gd php-mbstring php-xml php-intl samba-client }
    php_pkgs << 'php-pdo' if dbtype == 'sqlite'
    php_pkgs << 'php-mysql' if dbtype == 'mysql'
    php_pkgs << 'php-pgsql' if dbtype == 'pgsql'
  end
  ssl_key_dir = '/etc/pki/tls/private'
  ssl_cert_dir = '/etc/pki/tls/certs'
when 'fedora', 'scientific', 'amazon'
  php_pkgs = %w{ php-gd php-mbstring php-xml php-intl samba-client }
  php_pkgs << 'php-pdo' if dbtype == 'sqlite'
  php_pkgs << 'php-mysql' if dbtype == 'mysql'
  php_pkgs << 'php-pgsql' if dbtype == 'pgsql'

  ssl_key_dir = '/etc/pki/tls/private'
  ssl_cert_dir = '/etc/pki/tls/certs'
else
  log('Unsupported platform, trying to guess packages.') { level :warn }
  php_pkgs = %w{ php-gd php-mbstring php-xml php-intl samba-client }
  php_pkgs << 'php-pdo' if dbtype == 'sqlite'
  php_pkgs << 'php-mysql' if dbtype == 'mysql'
  php_pkgs << 'php-pgsql' if dbtype == 'pgsql'

  ssl_key_dir = node['owncloud']['www_dir']
  ssl_cert_dir = node['owncloud']['www_dir']
end

#==============================================================================
# Initialize autogenerated passwords
#==============================================================================

::Chef::Recipe.send(:include, Opscode::OpenSSL::Password)

if Chef::Config[:solo]
  if node['owncloud']['config']['dbpassword'].nil? and node['owncloud']['config']['dbtype'] != 'sqlite'
    Chef::Application.fatal!('You must set owncloud\'s database password in chef-solo mode.')
  end
  if node['owncloud']['admin']['pass'].nil?
    Chef::Application.fatal!('You must set owncloud\'s admin password in chef-solo mode.')
  end
else
  unless node['owncloud']['config']['dbtype'] == 'sqlite'
    node.set_unless['owncloud']['config']['dbpassword'] = secure_password
  end
  node.set_unless['owncloud']['admin']['pass'] = secure_password
  node.save
end

#==============================================================================
# Install PHP
#==============================================================================

include_recipe 'php'

php_pkgs.each do |pkg|
  package pkg do
    action :install
  end
end

#==============================================================================
# Set up database
#==============================================================================

case node['owncloud']['config']['dbtype']
when 'sqlite'
  # With SQLite the table prefix must be oc_
  node.override['owncloud']['config']['dbtableprefix'] = 'oc_'
when 'mysql'
  if %w{ localhost 127.0.0.1 }.include?(node['owncloud']['config']['dbhost'])
    # Install MySQL
    include_recipe 'mysql::server'
    include_recipe 'database::mysql'

    mysql_connection_info = {
      :host => 'localhost',
      :username => 'root',
      :password => node['mysql']['server_root_password']
    }

    mysql_database node['owncloud']['config']['dbname'] do
      connection mysql_connection_info
      action :create
    end

    mysql_database_user node['owncloud']['config']['dbuser'] do
      connection mysql_connection_info
      database_name node['owncloud']['config']['dbname']
      host 'localhost'
      password node['owncloud']['config']['dbpassword']
      privileges [:all]
      action :grant
    end
  end
when 'pgsql'
  if %w{ localhost 127.0.0.1 }.include?(node['owncloud']['config']['dbhost'])
    # Install PostgreSQL
    include_recipe 'postgresql::server'
    include_recipe 'database::postgresql'

    postgresql_connection_info = {
      :host => 'localhost',
      :username => 'postgres',
      :password => node['postgresql']['password']['postgres']
    }

    postgresql_database node['owncloud']['config']['dbname'] do
      connection postgresql_connection_info
      action :create
    end

    postgresql_database_user node['owncloud']['config']['dbuser'] do
      connection postgresql_connection_info
      host 'localhost'
      password node['owncloud']['config']['dbpassword']
      action :create
    end

    postgresql_database_user node['owncloud']['config']['dbuser'] do
      connection postgresql_connection_info
      database_name node['owncloud']['config']['dbname']
      host 'localhost'
      password node['owncloud']['config']['dbpassword']
      privileges [:all]
      action :grant
    end
  end
else
  Chef::Application.fatal!("Unsupported database type: #{node['owncloud']['config']['dbtype']}")
end

#==============================================================================
# Set up mail transfer agent
#==============================================================================

if  node['owncloud']['config']['mail_smtpmode'].eql?('sendmail') and
    node['owncloud']['install_postfix']

  include_recipe 'postfix::default'
end

#==============================================================================
# Download and extract ownCloud
#==============================================================================

directory node['owncloud']['www_dir']

basename = ::File.basename(node['owncloud']['download_url'])
local_file = ::File.join(Chef::Config[:file_cache_path], basename)

http_request 'HEAD owncloud' do
  message ''
  url node['owncloud']['download_url']
  action :head
  if File.exists?(local_file)
    headers 'If-Modified-Since' => File.mtime(local_file).httpdate
  end
  notifies :create, 'remote_file[download owncloud]', :immediately
end

remote_file 'download owncloud' do
  source node['owncloud']['download_url']
  path local_file
  action :nothing
  notifies :run, 'execute[extract owncloud]', :immediately
end

execute 'extract owncloud' do
  command "tar xfj '#{local_file}' --no-same-owner"
  cwd node['owncloud']['www_dir']
  action :nothing
end

[
  ::File.join(node['owncloud']['dir'], 'apps'),
  ::File.join(node['owncloud']['dir'], 'config'),
  node['owncloud']['data_dir']
].each do |dir|
  directory dir do
    owner node['apache']['user']
    group node['apache']['group']
    mode 00750
    action :create
  end
end

#==============================================================================
# Set up webserver
#==============================================================================

include_recipe 'apache2::default'
include_recipe 'apache2::mod_php5'

# Disable default site
apache_site 'default' do
  enable false
end

# Create virtualhost for ownCloud
web_app 'owncloud' do
  template 'vhost.erb'
  docroot node['owncloud']['dir']
  server_name node['owncloud']['server_name']
  port '80'
  enable true
end

# Enable ssl
if node['owncloud']['ssl']
  include_recipe 'apache2::mod_ssl'

  cert = OwnCloud::Certificate.new(node['owncloud']['server_name'])
  ssl_key_path = ::File.join(ssl_key_dir, 'owncloud.key')
  ssl_cert_path = ::File.join(ssl_cert_dir, 'owncloud.pem')

  # Create ssl certificate key
  file 'owncloud.key' do
    path ssl_key_path
    owner 'root'
    group 'root'
    mode 00600
    content cert.key
    action :create_if_missing
    notifies :create, 'file[owncloud.pem]', :immediately
  end

  # Create ssl certificate
  file 'owncloud.pem' do
    path ssl_cert_path
    owner 'root'
    group 'root'
    mode 00644
    content cert.cert
    action :nothing
  end

  # Create SSL virtualhost
  web_app 'owncloud-ssl' do
    template 'vhost.erb'
    docroot node['owncloud']['dir']
    server_name node['owncloud']['server_name']
    port '443'
    ssl_key ssl_key_path
    ssl_cert ssl_cert_path
    enable true
  end
end

#==============================================================================
# Initialize configuration file and install ownCloud
#==============================================================================

# create autoconfig.php for the installation
template 'autoconfig.php' do
  path ::File.join(node['owncloud']['dir'], 'config', 'autoconfig.php')
  source 'autoconfig.php.erb'
  owner node['apache']['user']
  group node['apache']['group']
  mode 00640
  variables(
    :dbtype => node['owncloud']['config']['dbtype'],
    :dbname => node['owncloud']['config']['dbname'],
    :dbuser => node['owncloud']['config']['dbuser'],
    :dbpass => node['owncloud']['config']['dbpassword'],
    :dbhost => node['owncloud']['config']['dbhost'],
    :dbprefix => node['owncloud']['config']['dbtableprefix'],
    :admin_user => node['owncloud']['admin']['user'],
    :admin_pass => node['owncloud']['admin']['pass'],
    :data_dir => node['owncloud']['data_dir']
  )
  not_if { ::File.exists?(::File.join(node['owncloud']['dir'], 'config', 'config.php')) }
  notifies :restart, 'service[apache2]', :immediately
  notifies :get, 'http_request[run setup]', :immediately
end

# install ownCloud
http_request 'run setup' do
  url 'http://localhost/'
  message ''
  action :nothing
end

# Apply the configuration on attributes to config.php
ruby_block 'apply config' do
  block do
    config_file = ::File.join(node['owncloud']['dir'], 'config', 'config.php')
    config = OwnCloud::Config.new(config_file)
    config.merge(node['owncloud']['config'])
    config.write
    unless Chef::Config[:solo]
      # store important options that where generated automatically by the setup
      node.set_unless['owncloud']['config']['passwordsalt'] = config['passwordsalt']
      node.set_unless['owncloud']['config']['instanceid'] = config['instanceid']
      node.save
    end
  end
end

#==============================================================================
# Enable cron for background jobs
#==============================================================================

if node['owncloud']['cron']['enabled'] == true
  cron 'owncloud cron' do
    user node['apache']['user']
    minute node['owncloud']['cron']['min']
    hour node['owncloud']['cron']['hour']
    day node['owncloud']['cron']['day']
    month node['owncloud']['cron']['month']
    weekday node['owncloud']['cron']['weekday']
    command "php -f '#{node['owncloud']['dir']}/cron.php' >> '#{node['owncloud']['data_dir']}/cron.log' 2>&1"
  end
else
  cron 'owncloud cron' do
    user node['apache']['user']
    command "php -f '#{node['owncloud']['dir']}/cron.php' >> '#{node['owncloud']['data_dir']}/cron.log' 2>&1"
    action :delete
  end
end
